
import java.lang.Math;

public class MatVec {
	
	/**
	 * Print the vector on standard output
	 * @param vec
	 */
	public static void printVec(double[] vec){
		System.out.print("(");
		for (int i = 0 ; i < vec.length ; i++ ){
			System.out.print( vec[i] + " " );
		}
		System.out.println(")");
	}

	/**
	 * Print the matrix on standard output
	 * mat[i][j] is assumed the ith raw, jth column
	 * @param mat
	 */
	public static void printMat(double[][] mat){
		System.out.println();
		for (int j = 0; j < mat.length; j++) {
			System.out.print("(");	
			for (int i = 0 ; i < mat[0].length ; i++ ){
				System.out.print( mat[j][i] + " " );
				}	
			System.out.println(")");
			}
		System.out.println();
	}
	
	/**
	 * auxilary function for norm2 norm3
	 * @param vec
	 * @param size
	 * @return the norm of the vector vec 
	 */
	private static double normX(double vec[], int size)
	{
		double normVec = 0;
		
		for (int i = 0; i < size; i++) {
			normVec += vec[i]*vec[i];
		}
		return Math.sqrt( normVec );
	}

	/** 
	 * @param vec
	 * @return the norm of vec, assuming it is a vector of R^2
	 */
	public static double norm2( double vec[] ){
		return normX(vec, 2);
	}
	
	/**
	 * @param vec
	 * @return the norm of vec, assuming it is a vector of R^3
	 */
	public static double norm3( double vec[] ){
		return normX(vec, 3);
	}
	
	/**
	 * auxilary function for multMat44Vec, multMat43Vec, multMat33Vec, multMat32Vec
	 * multiplies a matrix of size (nbR, nbC) with a vector of size nbC 
	 * @param mat
	 * @param nbL
	 * @param nbR
	 * @param vec
	 * @return the resulting vector (of size nbR)
	 */
	private static double[] multMatXXVec(double[][] mat, int nbL, int nbC, double[] vec ){
		double[] resu = new double[nbC]; 

		for (int i = 0; i < nbC; i++) {
			resu[i] = 0;
			for (int j = 0; j < nbL; j++) {
				resu[i] += mat[i][j]*vec[j];
			}
		}
		
		return resu;
	}
	
	/**
	 * multiplies two 4x4 matrices
	 * @param mat1
	 * @param mat2
	 * @return mat1.mat2
	 */
	public static double[][] multMat44Mat(double[][] mat1, double[][] mat2 ){
		double[][] matResu = new double[4][4];
		
		for (int i = 0; i < matResu.length; i++) {
			for (int j = 0; j < matResu[0].length; j++) {
				matResu[i][j] = 0;
				for (int k = 0; k < 4; k++) {
					matResu[i][j] += mat1[i][k]*mat2[k][j];					
				}
			}
		}
		return matResu;	
	}
	
	/**
	 * multiplies a 3x3 matrix by a vector
	 * @param mat
	 * @param vec
	 * @return mat.vec
	 */
	public static double[] multMat33Vec(double[][] mat, double[] vec ){
		return multMatXXVec( mat, 3, 3, vec);
	}

	/**
	 * multiplies a 3x4 matrix by a vector
	 * @param mat
	 * @param vec
	 * @return mat.vec
	 */
	public static double[] multMat34Vec(double[][] mat, double[] vec ){
		return multMatXXVec( mat, 3, 4, vec);
	}

	/**
	 * multiplies a 4x4 matrix by a vector
	 * @param mat
	 * @param vec
	 * @return mat.vec
	 */
	public static double[] multMat44Vec(double[][] mat, double[] vec ){
		return multMatXXVec( mat, 4, 4, vec);
	}

	/**
	 * multiplies a 2x3 matrix by a vector
	 * @param mat
	 * @param vec
	 * @return mat.vec
	 */
	public static double[] multMat23Vec(double[][] mat, double[] vec ){
		return multMatXXVec( mat, 2, 3, vec);
	}
	
	/**
	 * vector addition
	 * @param vec1
	 * @param vec2
	 * @return vec1 + vec2
	 */
	public static double[] addVec3(double[] vec1, double[] vec2){
		double[] vec3 = new double[3];
	
		for (int i = 0; i < 3; i++) {	
			vec3[i] = vec1[i] + vec2[i]; 
		}

		return vec3;	
	}

	/**
	 * vector soustraction
	 * @param vec1
	 * @param vec2
	 * @return vec1 - vec2
	 */
	public static double[] sousVec3(double[] vec1, double[] vec2){
		double[] vec3 = new double[3];
	
		for (int i = 0; i < 3; i++) {	
			vec3[i] = vec1[i] - vec2[i]; 
		}

		return vec3;	
	}
	
  /**
	 * linear combination
   * @param a 
	 * @param x
   * @param b
	 * @param y if null considered as zero
	 * @return a x + b y
   */
  private static final double [] theZero = {0,0,0,0} ;
  public static double[] aXAndBY(double a , double[] x, double b, double[] y){
    double[] ret = new double[3];
    if (y == null) {
      y = theZero ;
    }
		for (int i = 0; i < 3; i++) {	
			ret[i] = (a * x[i])  +  (b *  y[i]); 
		}
		return ret;	
  }



	/**
	 * vector product assuming Vec1 and Vec2 are of length 3
	 * @param vec1
	 * @param vec2
	 * @return vec1^vec2
	 */
	public static double[] prodVec(double[] vec1, double[] vec2){
		double[] vec3 = new double[3];
	
		for (int i = 0; i < 3; i++) {	
		vec3[i] = vec1[(i+1)%3]*vec2[(i+2)%3] - vec2[(i+1)%3]*vec1[(i+2)%3]; 
		}

		return vec3;	
	}
	
	/**
	 * dot product for vector of length 3
	 * @param vec1
	 * @param vec2
	 * @return
	 */
	public static double dotProd3(double[] vec1, double[] vec2){
		double resu = 0;
		
		for (int i = 0; i < 3; i++) {	
			resu += vec1[i]*vec2[i]; 
		}

		return resu;	
	}
	
	
}
