
import java.io.* ;

public class MeshData {

  double vertices[] ;
  int faces [] ;
  double colors [] ;

  private static String nextLine(BufferedReader in) throws Exception  {
    String r = in.readLine();

    while (r.matches("\\s*#.*")) {
      r = in.readLine() ;
    }
    return r ;
  }
  
  public static MeshData md;

  public MeshData(String filename) throws Exception {
    
    md = this;
    
    InputStream is = getClass().getResourceAsStream(filename);
    //BufferedReader in = new BufferedReader(new FileReader(filename));
    BufferedReader in = new BufferedReader(new InputStreamReader(is));
    
 
    String r = nextLine(in);
    if ( ! r.equals("OFF")) {
      throw new IOException("Not an OFF file !");
    }

    //System.out.println("Cur line: " + r);
    r = nextLine(in);
    //System.out.println("Cur line: " + r);

    String [] sar = r.split("\\s+");

    int verts_nb = new Integer(sar[0]).intValue();
    int faces_nb = new Integer(sar[1]).intValue();
    int edges_nb = new Integer(sar[2]).intValue();

    //System.out.println(" Verts : " + verts_nb + ", faces: " + faces_nb);
    vertices = new double[3*verts_nb];
    faces = new int[3*faces_nb];
    colors = new double[3*verts_nb];
    for (int i = 0 ; i < verts_nb ; i++ ){

      r = nextLine(in);
      sar = r.split("\\s+");

      vertices[ 3 * i + 0 ] = new Double(sar[0]).doubleValue();
      vertices[ 3 * i + 1 ] = new Double(sar[1]).doubleValue();
      vertices[ 3 * i + 2 ] = new Double(sar[2]).doubleValue();
      colors  [ 3 * i + 0 ] = new Double(sar[3]).doubleValue();
      colors  [ 3 * i + 1 ] = new Double(sar[4]).doubleValue();
      colors  [ 3 * i + 2 ] = new Double(sar[5]).doubleValue();

    }
    for (int i = 0 ; i < faces_nb ; i++ ){

      r = nextLine(in);
      sar = r.split("\\s+");

      int en = new Integer(sar[0]).intValue();
      if (en != 3) {
        throw new IOException("Sorry I only support triangular meshes...");
      }
      faces[ 3 * i + 0 ] =  new Integer(sar[1]).intValue();
      faces[ 3 * i + 1 ] =  new Integer(sar[2]).intValue();
      faces[ 3 * i + 2 ] =  new Integer(sar[3]).intValue();

    }
    in.close() ;

  }
 
  public double[] getNormals(){ // une normale par sommet
	  double[] normals = new double[vertices.length];
	  double[] A = new double[3];
	  double[] B = new double[3];
	  double[] C = new double[3];
	  double[] normal = new double[3];
	  double normN;
	  
	  // calcule une normale par face
	  for (int i = 0; i < faces.length; i=i+3) {
		  
		  System.arraycopy(vertices,faces[i+0]*3,A,0,3);
		  System.arraycopy(vertices,faces[i+1]*3,B,0,3);
		  System.arraycopy(vertices,faces[i+2]*3,C,0,3);
		  
		  normal = MatVec.prodVec(MatVec.sousVec3(B, A), MatVec.sousVec3(C, A));
		  normN = MatVec.norm3(normal);
		  for (int j = 0; j < 3; j++) {
			  normal[j] = normal[j]/normN;
		  }	
		  
		  /*  System.out.println("normale" + (i/3)); ;
		      System.out.println(
		        normal[  0 ] + " " +
		        normal[  1 ] + " " +
		        normal[  2 ] );
		 */
		  
		  // ajoute la normale calculee a chq sommet de la face
		  for (int j = 0; j < 3; j++) { // pr chq sommet
			  for (int k = 0; k < 3; k++) { // pr chq coord
				  normals[3*faces[i+j]+k] += normal[k];
			  }
		  }
	  }
	  
	  // on fait un tour des normals pour les rendre unitaires
	  for (int i = 0; i < normals.length; i=i+3) {
		  System.arraycopy(normals, i, normal, 0, 3);
		  normN = MatVec.norm3(normal);
		  for (int k = 0; k < 3; k++) { // pr chq coord
			  normals[i+k] = normal[k]/normN;		  
		  }
	  }
	  
	  return normals;
  }

  public double[] getVertices() {
    return vertices;
  }
  public int [] getFaces() {
    return faces;
  }
  public double [] getColors() {
    return colors;
  }


}
